package org.apache.ojb.broker.metadata.fieldaccess;

/* Copyright 2003-2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.apache.ojb.broker.metadata.AnonymousFieldDescriptor;
import org.apache.ojb.broker.metadata.ClassDescriptor;
import org.apache.ojb.broker.metadata.CollectionDescriptor;
import org.apache.ojb.broker.metadata.DescriptorRepository;
import org.apache.ojb.broker.metadata.FieldDescriptor;
import org.apache.ojb.broker.metadata.ObjectReferenceDescriptor;

import java.lang.reflect.Field;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

/**
 * Helper class for anonymous fields.
 *
 * @author Houar TINE
 * @version $Id: AnonymousPersistentFieldHelper.java,v 1.7 2004/04/04 23:53:35 brianm Exp $ 
 */
public class AnonymousPersistentFieldHelper
{
    /**
     * An iterative version of org.apache.ojb.broker.metadata.fieldaccess.PersistentFieldxxx
     * This implementastion is more efficient than recursive version
     * If fieldname not exist null is returned
     */
    public static Field getFieldIterative(final Class c, final String fieldname)
    {
        Field f = null;
        Class clazz = c;
        do
        {
            try
            {
                f = clazz.getDeclaredField(fieldname);
                break;
            }
            catch (NoSuchFieldException e)
            {
                // ignore
            }
            clazz = clazz.getSuperclass();
        }
        while (clazz != Object.class);
        return f;
    }

    public static Vector getPkFieldNames(final ClassDescriptor cld)
    {
        FieldDescriptor[] pks = cld.getPkFields();
        Vector v = new Vector(pks.length);
        for (int i = pks.length - 1; i >= 0; i--)
        {
            v.add(pks[i].getPersistentField().getName());
        }
        return v;
    }

    public static void computeInheritedPersistentFields(final DescriptorRepository repository)
    {
        Map descriptorTable = repository.getDescriptorTable();
        final String OBJECT = Object.class.getName();
        for (Iterator iter = descriptorTable.keySet().iterator(); iter.hasNext();)
        {
            String key = (String) iter.next();
            ClassDescriptor cld = (ClassDescriptor) descriptorTable.get(key);
            String baseClass = cld.getBaseClass();
            if (null != baseClass)
            {
                Vector v = new Vector();
                do
                {
                    ClassDescriptor tempCld = repository.getDescriptorFor(baseClass);
                    FieldDescriptor[] fields = tempCld.getFieldDescriptions();
                    Vector pkFields = getPkFieldNames(tempCld);
                    for (int i = fields.length - 1; i >= 0; i--)
                    {
                        FieldDescriptor f = fields[i];
                        if (!(f instanceof AnonymousFieldDescriptor))
                        {
                            PersistentField pf = f.getPersistentField();
                            if (!pkFields.contains(pf.getName()))
                            {
                                v.add(f.getPersistentField());
                            }
                        }
                    }
                    Vector ords = tempCld.getObjectReferenceDescriptors();
                    for (int i = ords.size() - 1; i >= 0; i--)
                    {
                        ObjectReferenceDescriptor ord = (ObjectReferenceDescriptor) ords.get(i);
                        PersistentField pf = ord.getPersistentField();
                        if (!(pf instanceof AnonymousPersistentFieldForInheritance))
                        {
                            v.add(pf);
                        }
                    }
                    Vector cds = tempCld.getCollectionDescriptors();
                    for (int i = cds.size() - 1; i >= 0; i--)
                    {
                        CollectionDescriptor cd = (CollectionDescriptor) cds.get(i);
                        v.add(cd.getPersistentField());
                    }
                    baseClass = tempCld.getBaseClass();
                }
                while ((null != baseClass) && !(baseClass.equals(OBJECT)));
                cld.setSuperPersistentFieldDescriptors(v);
            }
        }
    }
}
