package org.apache.ojb.broker.util.sequence;

/* Copyright 2002-2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.apache.ojb.broker.accesslayer.JdbcAccess;
import org.apache.ojb.broker.metadata.ClassDescriptor;
import org.apache.ojb.broker.metadata.FieldDescriptor;

/**
 * SequenceManagers are responsible for creating new unique
 * ID's - unique accross all extent objects.
 * There are some standard sequence manager implementations in
 * this package.
 * SequenceManager objects are obtained from a Factory called
 * {@link SequenceManagerFactory}.
 * This Factory can be configured to provide instances of user defined
 * implementors of this interface.
 *
 * SequenceManagers should be aware of extents, that
 * is: if you ask for an uid for an Interface (more exact for one implementor class)
 * with several implementor classes, or a baseclass with several subclasses the returned uid
 * should be unique accross all tables representing objects of the extent in question.
 *
 * @author <a href="mailto:thma@apache.org">Thomas Mahler<a>
 * @author <a href="mailto:armin@codeAuLait.de">Armin Waibel</a>
 * @version $Id: SequenceManager.java,v 1.10 2004/04/04 23:53:38 brianm Exp $
 *
 */
public interface SequenceManager
{
    /**
     * Returns a unique object for the given field attribute.
     * The returned value takes in account the jdbc-type
     * and the FieldConversion.sql2java() conversion defined for <code>field</code>.
     * The returned object is unique accross all tables in the extent
     * of class the field belongs to.
     */
    public Object getUniqueValue(FieldDescriptor field) throws SequenceManagerException;


    /**
     * This is to support auto_increment columns on the db side
     * <p>
     * high/low and other implementations may ignore this method.
     * <p>
     * Travis Reeder
     *
     * @param cld
     * @param obj
     */
    public void afterStore(JdbcAccess dbAccess, ClassDescriptor cld, Object obj) throws SequenceManagerException;

    /**
     * This is used for database auto_increment/identity columns to set the reference
     * foreign keys after they've
     * been assigned by the db.
     * <p>
     * This can be ignored by high/low implementations
     * <p>
     * Travis Reeder
     * @param obj
     * @param cld
     */
    public void setReferenceFKs(Object obj, ClassDescriptor cld) throws SequenceManagerException;
}
