package org.apache.ojb.odmg;

import org.apache.ojb.broker.util.configuration.Configuration;
import org.apache.ojb.broker.util.logging.Logger;
import org.apache.ojb.broker.util.logging.LoggerFactory;
import org.odmg.TransactionNotInProgressException;

/* Copyright 2002-2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
public class LocalTxManager implements OJBTxManager
{
    private static Logger log = LoggerFactory.getLogger(LocalTxManager.class);

    /**
     * Internal table which provides mapping between threads and transactions.
     * This is required because a Thread can join a Transaction already in
     * progress.
     * If the thread joins a transaction, then "getTransaction()" should return
     * the apropriate one.  The only way we can ensure that is by keeping hold
     * of the txTable.
     */
    private static TransactionTable tx_table = new TransactionTable();

    public LocalTxManager()
    {

    }

    /**
     * Returns the current transaction for the calling thread.
     * @throws org.odmg.TransactionNotInProgressException {@link org.odmg.TransactionNotInProgressException} if no transaction was found.
     */
    public TransactionImpl getCurrentTransaction()
    {
        TransactionImpl tx = (TransactionImpl) tx_table.get(Thread.currentThread());
        if (tx == null)
        {
            throw new TransactionNotInProgressException(
                    "Calling method needed transaction, but no transaction found for current thread :-(");
        }
        return tx;
    }

    /**
     * Returns the current transaction for the calling thread or <code>null</code>
     * if no transaction was found.
     */
    public TransactionImpl getTransaction()
    {
        return (TransactionImpl) tx_table.get(Thread.currentThread());
    }

	/**
	 * add the current transaction to the map key'd by the calling thread.
	 */
    public void registerTx(TransactionImpl tx)
    {
        tx_table.put(Thread.currentThread(), tx);
    }

	/**
	 * remove the current transaction from the map key'd by the calling thread.
	 */
    public void deregisterTx(Object token)
    {
        tx_table.remove(Thread.currentThread());
    }
    
	/**
	 * included to keep interface contract consistent.
	 */
    public void abortExternalTx(TransactionImpl odmgTrans)
    {
    	/**
    	 * no op
    	 */
    }
    
    public void configure(Configuration config)
    {
    	
    }
    
}
