package org.apache.ojb.tutorial2;

/* Copyright 2002-2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.odmg.Implementation;

import java.io.BufferedReader;
import java.io.InputStreamReader;

/**
 * Base class for the use cases of tutorial 2.
 */
public abstract class AbstractUseCase implements UseCase
{
    /** The ODMG handle used for database operations */
    protected Implementation odmg;

    /**
     * Creates a new instance of the use case. Is intended to be called
     * by sub classes.
     * 
     * @param odmg The odmg implementation to use
     */
    public AbstractUseCase(Implementation odmg)
    {
        this.odmg = odmg;
    }

    /**
     * Reads a single line from stdin and returns it.
     * 
     * @param message The message to print
     * @return The text entered by the user
     */
    protected String readLineWithMessage(String message)
    {
        System.out.print(message + " ");

        try
        {
            BufferedReader rin = new BufferedReader(new InputStreamReader(System.in));

            return rin.readLine();
        }
        catch (Exception e)
        {
            return "";
        }
    }
}
