package org.apache.ojb.tutorial5;

/* Copyright 2002-2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.Collection;

import javax.jdo.PersistenceManager;
import javax.jdo.PersistenceManagerFactory;
import javax.jdo.Query;
import javax.jdo.Transaction;

/**
 * Use case for deleting a product from the database.
 */
public class UCDeleteProduct extends AbstractUseCase
{
    /**
     * Creates a new delete use case instance.
     * 
     * @param factory The factory for getting persistence managers
     */
    public UCDeleteProduct(PersistenceManagerFactory factory)
    {
        super(factory);
    }

    /**
     * Returns a description of this use case.
     * 
     * @return A description of the use case
     */
    public String getDescription()
    {
        return "Delete a product entry";
    }

    /**
     * Performs this use case.
     */
    public void apply()
    {
        String      in = readLineWithMessage("Delete Product with id:");
        int         id = Integer.parseInt(in);
        Transaction tx = null;

        try
        {
			// 1. start transaction
            PersistenceManager manager = factory.getPersistenceManager();   

            tx = manager.currentTransaction();       
			tx.begin();

			// 2. Build a query to look up product by the id            
			Query query = manager.newQuery(Product.class, "id == " + id);     
            
			// 3. execute query  
			Collection result = (Collection) query.execute();
			
			// 4. if no matching product was found, print a message
			if (result.size() == 0)
			{
				System.out.println("did not find a Product with id=" + id);
				tx.rollback();
				manager.close();
				return;		
			}
			// 5. if a matching product was found, delete it
			else
			{
				Product toBeDeleted = (Product) result.iterator().next();        	

                manager.deletePersistent(toBeDeleted);
	            tx.commit();
            	manager.close();
			}
        }
        catch (Throwable t)
        {
            // rollback in case of errors
            tx.rollback();
            t.printStackTrace();
        }
    }
}
