/**
 * Author: Matthew Baird
 * mattbaird@yahoo.com
 */
package org.apache.ojb.broker;

import junit.framework.TestCase;
import org.apache.ojb.broker.query.*;
import org.apache.ojb.junit.PBTestCase;

import java.util.Iterator;

/**
 * tests a bidirectional association A<-->B
 * @see org.apache.ojb.odmg.BidirectionalAssociationTest for equivalent test in ODMG
 */
public class BidirectionalAssociationTest extends PBTestCase
{
    public static void main(String[] args)
    {
        String[] arr = {BidirectionalAssociationTest.class.getName()};
        junit.textui.TestRunner.main(arr);
    }

    public BidirectionalAssociationTest(String name)
    {
        super(name);
    }

    public void setUp() throws Exception
    {
        super.setUp();
        testCreateWithUpdate();
    }

    public void tearDown() throws Exception
    {
        try
        {
            deleteAllA();
            deleteAllB();
        }
        finally
        {
            super.tearDown();
        }

    }

    public void testCreateWithUpdate()
    {
        if (broker.isInTransaction())
            broker.abortTransaction();
        broker.beginTransaction();
        BidirectionalAssociationObjectA a = new BidirectionalAssociationObjectA();
        a.setPk("A" + System.currentTimeMillis());
        BidirectionalAssociationObjectB b = new BidirectionalAssociationObjectB();
        b.setPk("B" + System.currentTimeMillis());
        broker.store(a);
        broker.store(b);
        /**
         * now set relations
         */
        b.setRelatedA(a);
        a.setRelatedB(b);
        /**
         * and update
         */
        broker.store(a);
        broker.store(b);
        broker.commitTransaction();
    }

    public void testGetA() throws Exception
    {
        if (broker.isInTransaction())
            broker.abortTransaction();
        Criteria crit = new Criteria();
        Query q;
        Iterator iter;
        q = QueryFactory.newQuery(BidirectionalAssociationObjectA.class, crit);
        iter = broker.getIteratorByQuery(q);
        BidirectionalAssociationObjectA temp = null;
        while (iter.hasNext())
        {
            temp = (BidirectionalAssociationObjectA) iter.next();
            if (temp.getRelatedB() == null)
            {
                fail("relatedB not found");
            }
        }
    }

    public void testGetB() throws Exception
    {
        if (broker.isInTransaction())
            broker.abortTransaction();
        Criteria crit = new Criteria();
        Query q;
        Iterator iter;
        q = QueryFactory.newQuery(BidirectionalAssociationObjectB.class, crit);
        iter = broker.getIteratorByQuery(q);
        BidirectionalAssociationObjectB temp = null;
        while (iter.hasNext())
        {
            temp = (BidirectionalAssociationObjectB) iter.next();
            if (temp.getRelatedA() == null)
            {
                fail("relatedA not found");
            }
        }
    }

    public void testDeleteA()
    {
        deleteAllA();
    }

    public void testDeleteB()
    {
        deleteAllB();
    }

    private void deleteAllA()
    {
        if (broker.isInTransaction())
            broker.abortTransaction();
        Criteria crit = new Criteria();
        Query q;
        Iterator iter;
        q = QueryFactory.newQuery(BidirectionalAssociationObjectA.class, crit);
        iter = broker.getIteratorByQuery(q);
        BidirectionalAssociationObjectA temp = null;
        broker.beginTransaction();
        while (iter.hasNext())
        {
            temp = (BidirectionalAssociationObjectA) iter.next();
            BidirectionalAssociationObjectB b = temp.getRelatedB();
            if (b != null)
            {
                b.setRelatedA(null);
                broker.store(b);
            }
            broker.delete(temp);
        }
        broker.commitTransaction();
    }

   private void deleteAllB()
    {
        if (broker.isInTransaction())
            broker.abortTransaction();
        Criteria crit = new Criteria();
        Query q;
        Iterator iter;
        q = QueryFactory.newQuery(BidirectionalAssociationObjectB.class, crit);
        iter = broker.getIteratorByQuery(q);
        BidirectionalAssociationObjectB temp = null;
        broker.beginTransaction();
        while (iter.hasNext())
        {
            temp = (BidirectionalAssociationObjectB) iter.next();
            BidirectionalAssociationObjectA a = temp.getRelatedA();
            if (a != null)
            {
                a.setRelatedB(null);
                broker.store(a);
            }
            broker.delete(temp);
        }
        broker.commitTransaction();
    }
}
