package org.apache.ojb.broker;

import java.io.Serializable;

import junit.framework.TestCase;
import org.apache.ojb.broker.platforms.Platform;
import org.apache.ojb.broker.platforms.PlatformHsqldbImpl;
import org.apache.ojb.junit.PBTestCase;

/**
 * This TestClass tests storimg and retrieving Objects with BLOB/CLOB Attributes.
 * Does currently not work with HSQLDB, as it doen't support Blobs/Clobs.
 */
public class BlobTest extends PBTestCase
{
    boolean skipTest;
    private static final String msg = "# Skip "+BlobTest.class.getName()+", DB does not support Blob/Clob #";

    public BlobTest(String name)
    {
        super(name);
    }

    public static void main(String[] args)
    {
        String[] arr = {BlobTest.class.getName()};
        junit.textui.TestRunner.main(arr);
    }

    public void setUp() throws Exception
    {
        super.setUp();

        Platform platform = broker.serviceConnectionManager().getSupportedPlatform();
        /*
        hsqldb does not support Blob/Clob, so we skip test for this DB
        */
        if(platform.getClass().equals(PlatformHsqldbImpl.class))
        {
            skipTest = true;
        }
    }

    public void testBlobInsertion() throws Exception
    {
        if(skipTest)
        {
            System.out.println(msg);
            return;
        }

        int size = 5000;

        ObjectWithBlob obj = new ObjectWithBlob();

        byte[] barr = new byte[size];
        char[] carr = new char[size];
        for (int i = 0; i < size; i++)
        {
            barr[i] = (byte) 'x';
            carr[i] = 'y';
        }

        // obj.setId(1); we use autoincrement
        obj.setBlob(barr);
        obj.setClob(new String(carr));
        broker.beginTransaction();
        broker.store(obj);
        broker.commitTransaction();
        broker.clearCache();

        Identity oid = new Identity(obj, broker);
        ObjectWithBlob obj1 = (ObjectWithBlob) broker.getObjectByIdentity(oid);

        assertEquals(obj.getBlob().length, obj1.getBlob().length);
        assertEquals(obj.getClob().length(), obj1.getClob().length());

    }

    public void testReadNullBlob()
    {
        if(skipTest)
        {
            System.out.println(msg);
            return;
        }

        ObjectWithBlob obj = new ObjectWithBlob();

        // obj.setId(1); we use autoincrement
        obj.setBlob(null);
        obj.setClob(null);
        broker.beginTransaction();
        broker.store(obj);
        broker.commitTransaction();
        broker.clearCache();

        Identity oid = new Identity(obj, broker);
        ObjectWithBlob obj1 = (ObjectWithBlob) broker.getObjectByIdentity(oid);

        assertEquals(null, obj1.getBlob());
        assertEquals(null, obj1.getClob());
    }


    //*******************************************************
    // inner class - test class
    //*******************************************************
    public static class ObjectWithBlob implements Serializable
    {
        private int id;

        private byte[] blob;

        private String clob;


        /**
         * Gets the blob.
         * @return Returns a byte[]
         */
        public byte[] getBlob()
        {
            return blob;
        }

        /**
         * Sets the blob.
         * @param blob The blob to set
         */
        public void setBlob(byte[] blob)
        {
            this.blob = blob;
        }

        /**
         * Gets the clob.
         * @return Returns a char[]
         */
        public String getClob()
        {
            return clob;
        }

        /**
         * Sets the clob.
         * @param clob The clob to set
         */
        public void setClob(String clob)
        {
            this.clob = clob;
        }

        /**
         * Gets the id.
         * @return Returns a int
         */
        public int getId()
        {
            return id;
        }

        /**
         * Sets the id.
         * @param id The id to set
         */
        public void setId(int id)
        {
            this.id = id;
        }
    }
}
