package org.apache.ojb.broker;

import java.net.URL;
import java.util.Iterator;
import java.util.List;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.ojb.broker.query.Criteria;
import org.apache.ojb.broker.query.QueryByCriteria;
import org.apache.ojb.broker.util.ClassHelper;

/**
 * Tests for extent aware path expressions
 * 
 * @author <a href="leandro@ibnetwork.com.br">Leandro Rodrigo Saad Cruz</a>
 * 
 * $Id: ExtentAwarePathExpressionsTest.java,v 1.7 2004/06/01 19:15:24 tomdz Exp $
 */
public class ExtentAwarePathExpressionsTest extends TestCase
{

    /** */
	private PersistenceBroker broker;
	
    /**
	 * 
	 */
	public ExtentAwarePathExpressionsTest()
	{
		super();
	}

    /**
     * You can run the test like a java app 
     */
	public static void main(String[] args)
	{
        //OBS: when running this test from eclipse, one must copy the database file to the directory where 
        //u checked out ojb from cvs
        URL url = ClassHelper.getResource("DB.script");
        System.out.println("Hsql database @ :" + url);
		String[] arr = {ExtentAwarePathExpressionsTest.class.getName()};
		junit.textui.TestRunner.main(arr);
	}

	/**
	 * @param testeName
	 */
	public ExtentAwarePathExpressionsTest(String testName)
	{
		super(testName);
	}

    public static Test suite()
    {
        TestSuite suite = new TestSuite();
        //suite.addTest(new ExtentAwarePathExpressionsTest("testAddPathClass"));
        suite.addTest(new ExtentAwarePathExpressionsTest("testAddPathClasses"));
        //suite.addTest(new ExtentAwarePathExpressionsTest("testNotNullPathElement"));
        //suite.addTest(new ExtentAwarePathExpressionsTest("testSetPathClass"));
        //suite.addTest(new ExtentAwarePathExpressionsTest("testWithoutHintClass1"));
        //suite.addTest(new ExtentAwarePathExpressionsTest("testWithoutHintClass2"));
        return suite;
        
    }

	public void setUp() throws PBFactoryException
	{
		broker = PersistenceBrokerFactory.defaultPersistenceBroker();
	}

	public void tearDown()
	{
		broker.close();
	}

	public void testWithoutHintClass1() throws Exception
	{
		Criteria criteria = new Criteria();
		criteria.addEqualTo("qualifiers.name", "Cars");
		QueryByCriteria query = new QueryByCriteria(News.class, criteria);

		broker.clearCache();
		List content = (List) broker.getCollectionByQuery(query);
		assertEquals(1, content.size());
	}

	public void testWithoutHintClass2() throws Exception
	{
		Criteria criteria = new Criteria();
		criteria.addLike("qualifiers.name", "%ers%");
		QueryByCriteria query = new QueryByCriteria(BaseContentImpl.class, criteria, true);

		broker.clearCache();
		List content = (List) broker.getCollectionByQuery(query);
		assertEquals(2, content.size());
	}

    public void testNotNullPathElement()
    {
        Criteria criteria = new Criteria();
        criteria.addNotNull("qualifiers.name");
        QueryByCriteria query = new QueryByCriteria(BaseContentImpl.class, criteria, true);
        List content = (List) broker.getCollectionByQuery(query);
        assertEquals(4,content.size());
    }
    
    public void testSetPathClass()
    {
        Criteria criteria = new Criteria();
        criteria.addNotNull("qualifiers.name");
        QueryByCriteria query = new QueryByCriteria(BaseContentImpl.class, criteria, true);
        query.setPathClass("qualifiers",Category.class);
        List content = (List) broker.getCollectionByQuery(query);
        assertEquals(2,content.size());
        for (Iterator iter = content.iterator(); iter.hasNext();)
        {
            BaseContentImpl element = (BaseContentImpl) iter.next();
            assertTrue(element.getId() <=3 && element.getId() >=2);
        }

    }
    
    public void testAddPathClass()
    {
        Criteria criteria = new Criteria();
        criteria.addNotNull("qualifiers.name");
        QueryByCriteria query = new QueryByCriteria(BaseContentImpl.class, criteria, true);
        query.addPathClass("qualifiers",Qualifier.class);
        query.addPathClass("qualifiers",Area.class);
        List content = (List) broker.getCollectionByQuery(query);
        assertEquals(1,content.size());
        assertEquals(10,((Paper)content.get(0)).getId());
    }


    /**
     * The order of criteria elements influences the use of parenthesis, 
     *  If we add the Like element before the NotNull one we get the wrong query 
     * Right clause : WHERE WHERE (A2.NAME IS NOT NULL  OR A2E1.NAME IS NOT NULL ) AND A0.HEADLINE LIKE  'Bra%
     * Wrong clause : WHERE (A0.HEADLINE LIKE  'Bra%' ) AND A2.NAME IS NOT NULL  OR A2E1.NAME IS NOT NULL 
     */
    public void testAddPathClasses()
    {
        Criteria criteria = new Criteria();
        
        criteria.addLike("headline","Bra%");
        criteria.addNotNull("qualifiers.name");
        QueryByCriteria query = new QueryByCriteria(BaseContentImpl.class, criteria, true);
        query.addPathClass("qualifiers",Qualifier.class);
        query.addPathClass("qualifiers",Topic.class);
        query.addPathClass("qualifiers",Category.class);
        
        List content = (List) broker.getCollectionByQuery(query);
        assertEquals(1,content.size());
        assertEquals(3,((News)content.get(0)).getId());
    }

}
