package org.apache.ojb.broker;

import java.io.Serializable;
import java.math.BigDecimal;

import junit.framework.TestCase;
import org.apache.ojb.broker.accesslayer.conversions.ConversionException;
import org.apache.ojb.broker.accesslayer.conversions.FieldConversion;
import org.apache.ojb.broker.query.Criteria;
import org.apache.ojb.broker.query.QueryFactory;

/**
 * Test using field conversions for PK values.
 *
 * @author <a href="mailto:om@ppi.de">Oliver Matz</a>
 * @version $Id: FieldConversionTest_3.java,v 1.4 2004/06/25 15:58:39 arminw Exp $
 */
public class FieldConversionTest_3 extends TestCase
{
    private PersistenceBroker broker;

    public static void main(String[] args)
    {
        String[] arr = {FieldConversionTest_3.class.getName()};
        junit.textui.TestRunner.main(arr);
    }

    public void setUp() throws PBFactoryException
    {
        broker = PersistenceBrokerFactory.defaultPersistenceBroker();
    }

    public void tearDown()
    {
        try
        {
            broker.clearCache();
            broker.close();
        }
        catch (PersistenceBrokerException e)
        {
        }
    }

    /**
     * store nested classes needed field conversion of a primary key field.
     */
    public void testStoreNestedNodes() throws Exception
    {
        //String strQuery = "select allNodes from " + Node.class.getName();
        long id = System.currentTimeMillis();
        Node node = new Node(id, null, true);
        Node child = new Node(id + 1, node, false);

        int before;

        broker.beginTransaction();
        before = broker.getCount(QueryFactory.newQuery(Node.class, (Criteria) null));

        broker.store(child);

        broker.commitTransaction();

        broker.beginTransaction();
        int after = broker.getCount(QueryFactory.newQuery(Node.class, (Criteria) null));
        broker.commitTransaction();

        assertFalse(after == 0);
        assertEquals(before + 2, after);
    }

    /**
     * store class needed field conversion of a primary key field.
     */
    public void testStoreNode() throws Exception
    {
        //String strQuery = "select allNodes from " + Node.class.getName();
        long id = System.currentTimeMillis();
        Node node = new Node(id, null, false);

        int before;

        broker.beginTransaction();
        before = broker.getCount(QueryFactory.newQuery(Node.class, (Criteria) null));

        broker.store(node);

        broker.commitTransaction();

        broker.beginTransaction();
        int after = broker.getCount(QueryFactory.newQuery(Node.class, (Criteria) null));
        broker.commitTransaction();

        assertFalse(after == 0);
        assertEquals(before + 1, after);
    }


    //****************************************************************************
    // inner class
    //****************************************************************************
    public static class LongToBigDecimalConversion implements FieldConversion
    {
        public LongToBigDecimalConversion()
        {
        }

        public Object javaToSql(Object source) throws ConversionException
        {
            Object ret;
            if (source instanceof Long)
            {
                ret = new BigDecimal(((Long) source).doubleValue());
            }
            else
            {
                throw new ConversionException(
                        "java-->sql, expected type was"+Long.class.getClass()+
                        ", found type "+source.getClass());
            }
            return ret;
        }

        public Object sqlToJava(Object source) throws ConversionException
        {
            Object ret;
            if (source instanceof BigDecimal)
            {
                ret = new Long(((BigDecimal) source).longValue());
            }
            else
            {
                throw new ConversionException(
                        "sql-->java, expected type was"+BigDecimal.class.getClass()+
                        ", found type "+source.getClass());
            }
            return ret;
        }
    }


    //****************************************************************************
    // inner class
    //****************************************************************************
    public static class Node implements Serializable
    {
        private long uid;  // primary key
        private long refId;
        private boolean nodeState;
        Node parent;

        public Node()
        {
        }

        public Node(long uid, Node parent, boolean nodeState)
        {
            this.uid = uid;
            this.parent = parent;
            this.nodeState = nodeState;
        }

        public long getUid()
        {
            return uid;
        }

        public void setUid(long uid)
        {
            this.uid = uid;
        }

        public boolean isNodeState()
        {
            return nodeState;
        }

        public void setNodeState(boolean nodeState)
        {
            this.nodeState = nodeState;
        }

        public long getRefId()
        {
            return refId;
        }

        public void setRefId(long refId)
        {
            this.refId = refId;
        }

        public Node getParent()
        {
            return parent;
        }

        public void setParent(Node parent)
        {
            this.parent = parent;
        }
    }
}
