package org.apache.ojb.broker;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.apache.ojb.broker.query.Criteria;
import org.apache.ojb.broker.query.Query;
import org.apache.ojb.broker.query.QueryFactory;
import org.apache.ojb.junit.PBTestCase;

/**
 * Test inheritance using multiple tables.
 *
 * @author <a href="mailto:arminw@apache.org">Armin Waibel</a>
 * @version $Id: InheritanceMultipleTableTest.java,v 1.7 2004/05/11 11:55:57 arminw Exp $
 */
public class InheritanceMultipleTableTest extends PBTestCase
{
    public static void main(String[] args)
    {
        junit.textui.TestRunner.main(new String[] {InheritanceMultipleTableTest.class.getName()});
    }

    public void testQuery()
    {
        String name = "testInsert" + System.currentTimeMillis();
        Manager m_1 = new Manager(name + "_manager_1");
        Manager m_2 = new Manager(name + "_manager_2");
        Manager m_3 = new Manager(name + "_manager_3");
        m_3.setDepartment("none");

        Executive ex_1 = new Executive(name+"_executive", "department_1", null);
        Executive ex_2 = new Executive(name+"_executive", "department_1", null);

        Employee em = new Employee(name+"_employee");

        broker.beginTransaction();
        broker.store(m_1);
        broker.store(m_2);
        broker.store(m_3);
        broker.store(ex_1);
        broker.store(ex_2);
        broker.store(em);
        broker.commitTransaction();

        broker.clearCache();
        Criteria crit = new Criteria();
        crit.addLike("name", name + "%");
        crit.addLike("department", "none");
        Query query = QueryFactory.newQuery(Manager.class, crit);
        Collection result = broker.getCollectionByQuery(query);
        assertEquals(1, result.size());

        crit = new Criteria();
        crit.addLike("name", name+"%");
        query = QueryFactory.newQuery(Employee.class, crit);
        result = broker.getCollectionByQuery(query);
        assertEquals(6, result.size());

        crit = new Criteria();
        crit.addLike("name", name+"%");
        query = QueryFactory.newQuery(Executive.class, crit);
        result = broker.getCollectionByQuery(query);
        assertEquals(5, result.size());

        crit = new Criteria();
        crit.addLike("name", name+"%");
        query = QueryFactory.newQuery(Manager.class, crit);
        result = broker.getCollectionByQuery(query);
        assertEquals(3, result.size());
    }

    private void prepareForQueryTests(String name)
    {
        Manager m_1 = new Manager(name + "_manager_1");
        Manager m_2 = new Manager(name + "_manager_2");
        Manager m_3 = new Manager(name + "_manager_3");
        m_3.setDepartment("none");
        Address a_1 = new Address("snob allee");
        m_1.setAddress(a_1);

        Executive ex_1 = new Executive(name+"_executive", "department_1", null);
        Executive ex_2 = new Executive(name+"_executive", "department_1", null);

        Employee em = new Employee(name+"_employee");
        Address a_2 = new Address("cockroaches valley");
        em.setAddress(a_2);

        broker.beginTransaction();
        broker.store(m_1);
        broker.store(m_2);
        broker.store(m_3);
        broker.store(ex_1);
        broker.store(ex_2);
        broker.store(em);
        broker.commitTransaction();
    }

    public void testQuery_InheritedObjects()
    {
        String name = "testQuery_InheritedObjects" + System.currentTimeMillis();
        prepareForQueryTests(name);
        broker.clearCache();

        Criteria crit = new Criteria();
        crit.addLike("name", name+"%");
        Query query = QueryFactory.newQuery(Employee.class, crit);
        Collection result = broker.getCollectionByQuery(query);
        assertEquals(6, result.size());
        for(Iterator iterator = result.iterator(); iterator.hasNext();)
        {
            Employee obj = (Employee) iterator.next();
            assertNotNull(obj.getName());
        }

        broker.clearCache();
        crit = new Criteria();
        crit.addLike("name", name+"%");
        query = QueryFactory.newQuery(Executive.class, crit);
        result = broker.getCollectionByQuery(query);
        assertEquals(5, result.size());
        for(Iterator iterator = result.iterator(); iterator.hasNext();)
        {
            Executive obj = (Executive) iterator.next();
            assertNotNull(obj.getName());
        }

        broker.clearCache();
        crit = new Criteria();
        crit.addLike("name", name+"%");
        query = QueryFactory.newQuery(Manager.class, crit);
        result = broker.getCollectionByQuery(query);
        assertEquals(3, result.size());
        for(Iterator iterator = result.iterator(); iterator.hasNext();)
        {
            Manager obj = (Manager) iterator.next();
            assertNotNull(obj.getName());
        }
    }

    public void testQuery_InheritedField()
    {
        String name = "testQuery_InheritedField" + System.currentTimeMillis();
        prepareForQueryTests(name);
        broker.clearCache();

        broker.clearCache();
        Criteria crit = new Criteria();
        crit.addLike("name", name + "%");
        crit.addLike("department", "none");
        Query query = QueryFactory.newQuery(Manager.class, crit);
        Collection result = broker.getCollectionByQuery(query);
        assertEquals(1, result.size());
    }

    public void testQuery_Reference()
    {
        String name = "testQuery_Reference" + System.currentTimeMillis();
        prepareForQueryTests(name);
        broker.clearCache();

        Criteria crit = new Criteria();
        crit.addLike("name", name + "%");
        crit.addLike("address.street", "%valley");
        Query query = QueryFactory.newQuery(Employee.class, crit);
        Collection result = broker.getCollectionByQuery(query);
        assertEquals(1, result.size());
        Employee emp = (Employee) result.iterator().next();
        assertNotNull(emp.getAddress());
        assertEquals("cockroaches valley", emp.getAddress().getStreet());
    }

    public void testQuery_InheritedReference_1()
    {
        String name = "testQuery_InheritedReference_1" + System.currentTimeMillis();
        prepareForQueryTests(name);
        broker.clearCache();

        Criteria crit = new Criteria();
        crit.addLike("name", name + "%");
        crit.addEqualTo("address.street", "snob allee");
        Query query = QueryFactory.newQuery(Manager.class, crit);
        Collection result = broker.getCollectionByQuery(query);
        assertEquals(1, result.size());
        Manager retManager = (Manager) result.iterator().next();
        assertNotNull(retManager);
        assertEquals(name + "_manager_1", retManager.getName());
        assertNotNull(retManager.getAddress());
        assertEquals("snob allee", retManager.getAddress().getStreet());
    }

    public void testQuery_InheritedReference_2()
    {
        String name = "testQuery_InheritedReference_2" + System.currentTimeMillis();
        prepareForQueryTests(name);
        broker.clearCache();

        Criteria crit = new Criteria();
        crit.addLike("name", name + "%");
        crit.addEqualTo("address.street", "snob allee");
        Query query = QueryFactory.newQuery(Executive.class, crit);
        Collection result = broker.getCollectionByQuery(query);
        assertEquals(1, result.size());

        Executive retManager = (Executive) result.iterator().next();
        assertNotNull(retManager);
        assertEquals(name + "_manager_1", retManager.getName());
   }

    public void testQuery_InheritedReference_3()
    {
        String name = "testQuery_InheritedReference_3" + System.currentTimeMillis();
        prepareForQueryTests(name);
        broker.clearCache();

        Criteria crit = new Criteria();
        crit.addLike("name", name + "%");
        crit.addEqualTo("address.street", "snob allee");
        Query query = QueryFactory.newQuery(Employee.class, crit);
        Collection result = broker.getCollectionByQuery(query);
        assertEquals(1, result.size());
    }

    public void testInsertQuery()
    {
        String name = "testInsert" + System.currentTimeMillis();
        Employee em1 = new Employee(name);
        Executive ex1 = new Executive(name, "department_1", null);
        Executive ex2 = new Executive(name, "department_2", null);
        ArrayList list = new ArrayList();
        list.add(ex1);
        list.add(ex2);
        Manager m1 = new Manager(name);
        m1.setExecutives(list);

        broker.beginTransaction();
        broker.store(em1);
        broker.store(m1);
        broker.commitTransaction();

        Identity m1_oid = new Identity(m1, broker);
        Identity ex1_oid = new Identity(ex1, broker);
        Identity em1_oid = new Identity(em1, broker);

        broker.clearCache();

        Employee newEm1 = (Employee) broker.getObjectByIdentity(em1_oid);
        Executive newEx1 = (Executive) broker.getObjectByIdentity(ex1_oid);
        Manager newM1 = (Manager) broker.getObjectByIdentity(m1_oid);

        assertEquals(em1, newEm1);
        assertEquals(ex1, newEx1);
        assertEquals(m1, newM1);
        assertEquals(name, newEx1.getName());
        assertEquals(name, newM1.getName());

        assertEquals(2, newM1.getExecutives().size());

        Criteria crit = new Criteria();
        crit.addEqualTo("name", name);
        Query queryEmployee = QueryFactory.newQuery(Employee.class, crit);
        Query queryExecutive = QueryFactory.newQuery(Executive.class, crit);
        Query queryManager = QueryFactory.newQuery(Manager.class, crit);

        Collection result = broker.getCollectionByQuery(queryEmployee);
        assertEquals(4, result.size());

        result = broker.getCollectionByQuery(queryExecutive);
        assertEquals(3, result.size());

        result = broker.getCollectionByQuery(queryManager);
        assertEquals(1, result.size());
    }

    public void testUpdate()
    {
        String name = "testUpdate" + System.currentTimeMillis();
        Employee em1 = new Employee("employee_" + name);
        Executive ex1 = new Executive("executive_" + name, "department_1", null);
        Executive ex2 = new Executive("executive_" + name, "department_2", null);
        ArrayList list = new ArrayList();
        list.add(ex1);
        list.add(ex2);
        Manager m1 = new Manager("manager_" + name);
        m1.setExecutives(list);

        broker.beginTransaction();
        broker.store(em1);
        broker.store(m1);
        broker.commitTransaction();

        Identity m1_oid = new Identity(m1, broker);
        Identity ex1_oid = new Identity(ex1, broker);
        Identity em1_oid = new Identity(em1, broker);

        broker.clearCache();

        Employee newEm1 = (Employee) broker.getObjectByIdentity(em1_oid);
        Executive newEx1 = (Executive) broker.getObjectByIdentity(ex1_oid);
        Manager newM1 = (Manager) broker.getObjectByIdentity(m1_oid);

        assertEquals(2, newM1.getExecutives().size());

        newEm1.setName("**updated_"+name);
        newM1.setName("**updated_"+name);
        ((Executive) newM1.getExecutives().get(0)).setName("**updated_"+name);

        broker.beginTransaction();
        broker.store(newEm1);
        broker.store(newM1);
        broker.store(newEx1);
        broker.commitTransaction();

        broker.clearCache();

        em1 = (Employee) broker.getObjectByIdentity(em1_oid);
        ex1 = (Executive) broker.getObjectByIdentity(ex1_oid);
        m1 = (Manager) broker.getObjectByIdentity(m1_oid);

        assertEquals(newEm1, em1);
        assertEquals(newEx1, ex1);
        assertEquals(newM1, m1);

        assertEquals(2, m1.getExecutives().size());
    }

    public void testDelete()
    {
        String name = "testUpdate" + System.currentTimeMillis();
        Employee em1 = new Employee("employee_" + name);
        Executive ex1 = new Executive("executive_" + name, "department_1", null);
        Executive ex2 = new Executive("executive_" + name, "department_2", null);
        ArrayList list = new ArrayList();
        list.add(ex1);
        list.add(ex2);
        Manager m1 = new Manager("manager_" + name);
        m1.setExecutives(list);

        broker.beginTransaction();
        broker.store(em1);
        broker.store(m1);
        broker.commitTransaction();

        Identity m1_oid = new Identity(m1, broker);
        Identity ex1_oid = new Identity(ex1, broker);
        Identity em1_oid = new Identity(em1, broker);

        broker.clearCache();

        Employee newEm1 = (Employee) broker.getObjectByIdentity(em1_oid);
        Executive newEx1 = (Executive) broker.getObjectByIdentity(ex1_oid);
        Manager newM1 = (Manager) broker.getObjectByIdentity(m1_oid);

        assertNotNull(newEm1);
        assertNotNull(newEx1);
        assertNotNull(newM1);
        assertEquals(2, newM1.getExecutives().size());

        broker.beginTransaction();
        broker.delete(newEm1);
        broker.delete(newEx1);
        broker.delete(newM1);
        broker.commitTransaction();

        newEm1 = (Employee) broker.getObjectByIdentity(em1_oid);
        newEx1 = (Executive) broker.getObjectByIdentity(ex1_oid);
        newM1 = (Manager) broker.getObjectByIdentity(m1_oid);

        assertNull(newEm1);
        assertNull(newEx1);
        assertNull(newM1);
    }

    //************************************************************
    // inner classes used for test
    //************************************************************
    public static class Manager extends Executive
    {
        private List executives;

        public Manager()
        {
        }

        public Manager(String name)
        {
            super(name,null, null);
        }

        public List getExecutives()
        {
            return executives;
        }

        public void setExecutives(List executives)
        {
            this.executives = executives;
        }
    }

    public static class Executive extends Employee
    {
        private String department;
        private Manager manager;

        public Executive()
        {
        }

        public Executive(String name, String department, Manager manager)
        {
            super(name);
            this.department = department;
            this.manager = manager;
        }

        public String getDepartment()
        {
            return department;
        }

        public void setDepartment(String department)
        {
            this.department = department;
        }

        public Manager getManager()
        {
            return manager;
        }

        public void setManager(Manager manager)
        {
            this.manager = manager;
        }

        public boolean equals(Object obj)
        {
            if(! (obj instanceof Executive))
            {
                return false;
            }
            Executive ex = (Executive) obj;
            return new EqualsBuilder().append(getId(), ex.getId())
                    .append(getName(), ex.getName())
                    .append(getDepartment(), ex.getDepartment()).isEquals();
        }
    }

    public static class Employee
    {
        private Integer id;
        private String name;
        private AddressIF address;

        public Employee()
        {
        }

        public Employee(String name)
        {
            this.name = name;
        }

        public Integer getId()
        {
            return id;
        }

        public void setId(Integer id)
        {
            this.id = id;
        }

        public AddressIF getAddress()
        {
            return address;
        }

        public void setAddress(AddressIF address)
        {
            this.address = address;
        }

        public String getName()
        {
            return name;
        }

        public void setName(String name)
        {
            this.name = name;
        }

        public boolean equals(Object obj)
        {
            if(! (obj instanceof Employee))
            {
                return false;
            }
            Employee em = (Employee) obj;
            return new EqualsBuilder().append(getId(), em.getId()).append(getName(), em.getName()).isEquals();
        }

        public String toString()
        {
            return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE, false, Employee.class);
        }
    }

    public static class Address implements AddressIF
    {
        private Integer id;
        private String street;

        public Address()
        {
        }

        public Address(String street)
        {
            this.street = street;
        }

        public Integer getId()
        {
            return id;
        }

        public void setId(Integer id)
        {
            this.id = id;
        }

        public String getStreet()
        {
            return street;
        }

        public void setStreet(String street)
        {
            this.street = street;
        }
    }

    public static interface AddressIF
    {
        public Integer getId();
        public void setId(Integer id);
        public String getStreet();
        public void setStreet(String street);
    }
}
