package org.apache.ojb.broker;

import java.util.Collection;

import junit.framework.TestCase;

import org.apache.ojb.broker.metadata.ClassDescriptor;
import org.apache.ojb.broker.metadata.FieldDescriptor;
import org.apache.ojb.broker.query.Criteria;
import org.apache.ojb.broker.query.Query;
import org.apache.ojb.broker.query.QueryByIdentity;
import org.apache.ojb.broker.query.QueryFactory;

/** This TestClass tests OJB facilities to work with polymorphism.
 */
public class PolymorphicExtents extends TestCase
{
	public static void main(String[] args)
	{
		String[] arr = { CLASS.getName()};
		junit.textui.TestRunner.main(arr);
	}

	PersistenceBroker broker;
	private static Class CLASS = PolymorphicExtents.class;
	private int testId = 88882;

	/**
	 * Insert the method's description here.
	 * Creation date: (24.12.2000 00:33:40)
	 */
	public PolymorphicExtents(String name)

	{
		super(name);
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (06.12.2000 22:15:33)
	 * @param id int
	 */
	protected Article createArticle(int id)
	{
		Article a = new Article();
		a.setArticleId(id);
		a.setArticleName("New Funny Article " + id);
		a.setIsSelloutArticle(true);
		a.setMinimumStock(100);
		a.setOrderedUnits(17);
		a.setPrice(0.45);
		a.setProductGroupId(1);
		a.setStock(234);
		a.setSupplierId(4);
		a.setUnit("bottle");
		ProductGroup tmpPG = new ProductGroup();
		tmpPG.setId(1);
		Identity pgID = new Identity(tmpPG, broker);
		ProductGroupProxy pgProxy = new ProductGroupProxy(broker.getPBKey(),pgID);
		a.setProductGroup(pgProxy);
		return a;
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (06.12.2000 21:58:53)
	 */
	public void setUp() throws PBFactoryException
	{
		broker = PersistenceBrokerFactory.defaultPersistenceBroker();
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (06.12.2000 21:59:14)
	 */
	public void tearDown()
	{

		try
		{
			broker.clearCache();
			broker.close();
		}
		catch (PersistenceBrokerException e)
		{
		}

	}

	/** TestThreadsNLocks query support for polymorphic extents*/
	public void testCollectionByQuery()
	{
        Criteria crit = new Criteria();
        crit.addEqualTo("articleName", "Hamlet");
        Query q = QueryFactory.newQuery(InterfaceArticle.class, crit);

        Collection result = broker.getCollectionByQuery(q);

        //System.out.println(result);

        assertNotNull("should return at least one item", result);
        assertTrue("should return at least one item", result.size() > 0);
}

	/**
	 * try to retrieve a polymorphic collection attribute
	 * (ProductGroup.allArticlesInGroup contains items
	 * of type TestThreadsNLocks.org.apache.ojb.broker.Article which forms an extent)
	 *  ProductGroup 5 contain items from table Artikel, BOOKS and CDS
	 */
	public void testCollectionRetrieval()
	{
		try
		{
			ProductGroup example = new ProductGroup();
			example.setId(5);

			ProductGroup group =
				(ProductGroup) broker.getObjectByQuery(QueryFactory.newQuery(example));

            // 7 Articles, 2 Books, 3 Cds
            assertEquals("check size",group.getAllArticles().size(),12);

		}
		catch (Throwable t)
		{
			fail(t.getMessage());
		}

	}

	/** TestThreadsNLocks EXTENT lookup: a collection with ALL objects in the Article extent*/
	public void testExtentByQuery()
	{
		try
		{
			// no criteria signals to omit a WHERE clause
			Criteria selectAll = null;
			Query q = QueryFactory.newQuery(InterfaceArticle.class, selectAll);

			Collection result = broker.getCollectionByQuery(q);

			//System.out.println("OJB proudly presents: The InterfaceArticle Extent\n" + result);

			assertNotNull("should return at least one item", result);
			assertTrue("should return at least one item", result.size() > 0);

		}
		catch (Throwable t)

			{
			System.out.println(t.getMessage());
			t.printStackTrace();
			fail(t.getMessage());
		}
	}

	/** TestThreadsNLocks to lookup items from extent classes*/
	public void testRetrieveObjectByIdentity()
	{
        Article example = new Article();
        example.setArticleId(200);
        // id not present in table ARTICLES but int table CDS
        Identity oid = new Identity(example, broker);

        InterfaceArticle result = (InterfaceArticle) broker.getObjectByIdentity(oid);

        assertNotNull("should find a CD-article", result);
        assertTrue("should be of type CdArticle", (result instanceof CdArticle));

        example = new Article();
        example.setArticleId(100);
        // id not present in table ARTICLES but int table CDS
        oid = new Identity(example, broker);

        result = (InterfaceArticle) broker.getObjectByIdentity(oid);

        assertNotNull("should find a Book-article", result);
        assertTrue("should be of type BookArticle", (result instanceof BookArticle));
	}

	/** try to load polymorphic references
	 * (OrderPosition.article is of type InterfaceArticle)
	 */
	public void testRetrieveReferences()
	{
		try
		{
			for (int i = 1; i < 4; i++)
			{
				OrderPosition tmp = new OrderPosition();
				tmp.setId(i);
				Identity oid = new Identity(tmp, broker);
				OrderPosition pos = (OrderPosition) broker.getObjectByIdentity(oid);
				//System.out.println(pos);
			}
		}
		catch (Throwable t)
		{
			fail(t.getMessage());
		}
	}

	/**
     * Commented out!!
     * testing the SequenceManagement with extents.
     *
     * This test doesn't work in versions >0.97,
     * because you can't get a unique id for an interface,
     * only for real objects.
     * There many test cases for sequence manager
     * in {@link org.apache.ojb.broker.sequence.SequenceManagerTest}
	 */
	public void XXXtestSequenceManager() throws Exception
	{

        // ask for a new unique ID in the InterfaceArticle extent
        // the UID must be unique accross tables ARTIKEL, BOOKS, CDS !
        int uid = 0; //broker.serviceSequenceManager().getUniqueId(
                //broker.getClassDescriptor(Article.class).getFieldDescriptorByName("articleId"));

        // now check if is really unique !
        Article tmp = new Article();
        tmp.setArticleId(uid);
        Query q = new QueryByIdentity(tmp);
        InterfaceArticle result = (InterfaceArticle) broker.getObjectByQuery(q);

        assertNull("" + uid + " should be unique", result);

        //System.out.println("next free UID for InterfaceArticle : " + uid);

        // the next id from the sequence should be uid + 1, even if we did not store any object using uid.
    	ClassDescriptor cld = broker.getClassDescriptor(InterfaceArticle.class);
        FieldDescriptor fld = cld.getFieldDescriptorByName("articleId");

        int nextUid = 1; //broker.serviceSequenceManager().getUniqueId(fld);
        assertEquals("nextUid - uid = 1 !!!", 1, nextUid - uid);

	}
}
