package org.apache.ojb.broker;

import junit.framework.TestCase;
import org.apache.ojb.broker.query.QueryByIdentity;

/** This TestClass tests OJB facilities to work with typed collections.
 */
public class TypedCollectionsTest extends TestCase
{
	public static void main(String[] args)
	{
		String[] arr = { CLASS.getName()};
		junit.textui.TestRunner.main(arr);
	}

	PersistenceBroker broker;
	private static Class CLASS = TypedCollectionsTest.class;

	/**
	 * Insert the method's description here.
	 * Creation date: (24.12.2000 00:33:40)
	 */
	public TypedCollectionsTest(String name)

	{
		super(name);
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (06.12.2000 22:15:33)
	 * @param id int
	 */
	protected Article createArticle(int id)
	{
		Article a = new Article();
		a.setArticleId(id);
		a.setArticleName("New Funny Article " + id);
		a.setIsSelloutArticle(true);
		a.setMinimumStock(100);
		a.setOrderedUnits(17);
		a.setPrice(0.45);
		a.setProductGroupId(1);
		a.setStock(234);
		a.setSupplierId(4);
		a.setUnit("bottle");
		ProductGroup tmpPG = new ProductGroup();
		tmpPG.setId(1);
		Identity pgID = new Identity(tmpPG, broker);
		ProductGroupProxy pgProxy = new ProductGroupProxy(broker.getPBKey(), pgID);
		a.setProductGroup(pgProxy);
		return a;
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (06.12.2000 21:58:53)
	 */
	public void setUp() throws PBFactoryException
	{
		broker = PersistenceBrokerFactory.defaultPersistenceBroker();
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (06.12.2000 21:59:14)
	 */
	public void tearDown()
	{

		try
		{
			broker.clearCache();
			broker.close();
		}
		catch (PersistenceBrokerException e)
		{
		}
	}

	/** Test support for 1-n relations modelled with Arrays*/
	public void testArray() throws Exception
	{
		int i;
        broker.beginTransaction();
        for (i = 1; i < 4; i++)
        {
            ProductGroupWithArray example = new ProductGroupWithArray();
            example.setId(i);
            ProductGroupWithArray group =
                (ProductGroupWithArray) broker.getObjectByQuery(new QueryByIdentity(example));
            assertEquals("should be equal", i, group.getId());
            //System.out.println(group + "\n\n");

            broker.delete(group);
            broker.store(group);
        }
        broker.commitTransaction();
	}

	/** TestThreadsNLocks support for modelling 1-n relations with typed collections*/
	public void testTypedCollection()
	{
        broker.beginTransaction();
        for (int i = 1; i < 4; i++)
        {
            ProductGroupWithTypedCollection example = new ProductGroupWithTypedCollection();
            example.setId(i);
            ProductGroupWithTypedCollection group =
                (ProductGroupWithTypedCollection) broker.getObjectByQuery(
                    new QueryByIdentity(example));
            assertEquals("should be equal", i, group.getId());
            //System.out.println(group + "\n\n");

            broker.delete(group);
            broker.store(group);
        }
        broker.commitTransaction();
	}
}
