package org.apache.ojb.broker.metadata;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.ClassUtils;
import org.apache.ojb.broker.OJBRuntimeException;
import org.apache.ojb.broker.PersistenceBroker;
import org.apache.ojb.broker.PersistenceBrokerFactory;
import org.apache.ojb.broker.Person;
import org.apache.ojb.broker.Project;
import org.apache.ojb.broker.Role;
import org.apache.ojb.broker.query.Query;
import org.apache.ojb.broker.query.QueryByCriteria;
import org.apache.ojb.broker.sequence.Repository;
import org.apache.ojb.broker.util.ClassHelper;
import org.apache.ojb.junit.JUnitExtensions;

/**
 *
 * @author <a href="mailto:armin@codeAuLait.de">Armin Waibel</a>
 * @version $Id: MetadataMultithreadedTest.java,v 1.7 2004/04/05 17:13:27 arminw Exp $
 */
public class MetadataMultithreadedTest extends JUnitExtensions.MultiThreadedTestCase
{
    // we change table name in test for target class
    private String newTestObjectString = "SM_TAB_MAX_AA";
    private Class targetTestClass = Repository.SMMaxA.class;

    private String oldTestObjectString;
    DescriptorRepository defaultRepository;

    public MetadataMultithreadedTest(String s)
    {
        super(s);
    }

    public static void main(String[] args)
    {
        String[] arr = {MetadataMultithreadedTest.class.getName()};
        junit.textui.TestRunner.main(arr);
    }

    private long getTotalMemory()
    {
        long result = Long.MAX_VALUE;
        // TODO: find a solution for this problem, or uncomment if we cancel 1.2 support
        // result = Runtime.getRuntime().maxMemory(); // not available in JDK 1.2
        return result;
    }

    protected void setUp() throws Exception
    {
        super.setUp();
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
        MetadataManager.getInstance().setEnablePerThreadChanges(false);
    }

    private String getTestObjectString()
    {
        return oldTestObjectString;
    }

    public void testRuntimeMetadataChanges() throws Exception
    {
        int loops = 7;
        int threads = 8;
        // need min 80% free memory after test campared with
        // beginning, else test fails
        int minimalFreeMemAfterTest = 80;

        PersistenceBroker broker = null;
        try
        {
            MetadataManager mm = MetadataManager.getInstance();
            // enable the per thread changes of metadata
            mm.setEnablePerThreadChanges(true);
            defaultRepository = mm.copyOfGlobalRepository();
            mm.setDescriptor(defaultRepository);

            ClassDescriptor cld;
            long memoryUseBeforeTest;
            long memoryUseAfterTest;
            try
            {
                // prepare for test
                long period = System.currentTimeMillis();
                broker = PersistenceBrokerFactory.defaultPersistenceBroker();
                cld = broker.getClassDescriptor(targetTestClass);

                // we manipulate the schema name of the class
                // thus we note the original value
                oldTestObjectString = cld.getFullTableName();
                broker.close();

                // cleanup JVM
                Runtime.getRuntime().gc();
                Thread.sleep(200);
                Runtime.getRuntime().gc();

                // start test
                long memory = Runtime.getRuntime().freeMemory();
                long totalMemory = getTotalMemory();

                int count = 0;
                for (int k = 0; k < loops; k++)
                {
                    TestCaseRunnable tct [] = new TestCaseRunnable[threads];
                    for (int i = 0; i < threads; i++)
                    {
                        if (i % 2 == 0)
                            tct[i] = new ThreadedUsingBroker(loops);
                        else
                            tct[i] = new GlobalUsingBroker(loops);
                    }
                    // run test classes
                    runTestCaseRunnables(tct);
                    ++count;
                    System.out.println("Free/total Memory after loop " + count + ":          "
                            + convertToMB(Runtime.getRuntime().freeMemory())
                            + "/" + convertToMB(getTotalMemory()) + "MB");
                }
                period = System.currentTimeMillis() - period;
                System.out.println(ClassUtils.getShortClassName(MetadataMultithreadedTest.class) + " take: "
                        + period + " ms for " + loops + " loops, creating each with " + threads + " threads");
                System.out.println("Free/total Memory before test:          "
                        + convertToMB(memory) + "/" + convertToMB(totalMemory) + "MB");
                Runtime.getRuntime().gc();
                Thread.sleep(200);
                Runtime.getRuntime().gc();
                Runtime.getRuntime().gc();

                memoryUseBeforeTest = convertToMB(memory);
                memoryUseAfterTest = convertToMB(Runtime.getRuntime().freeMemory());
                System.out.println("Free/total Memory after test and gc:   "
                        + memoryUseAfterTest
                        + "/" + convertToMB(getTotalMemory()) + "MB");
                System.out.println("Do cleanup now ...");
            }
            finally
            {
                MetadataManager.getInstance().setEnablePerThreadChanges(false);
            }
            // get new PB instance
            broker = PersistenceBrokerFactory.defaultPersistenceBroker();
            cld = broker.getClassDescriptor(targetTestClass);
            String name = cld.getFullTableName();
            assertEquals(oldTestObjectString, name);
            assertFalse(MetadataManager.getInstance().isEnablePerThreadChanges());
            double d = ((double) memoryUseAfterTest) / ((double) memoryUseBeforeTest);
            int result = (int) (d * 100);
            if (result < minimalFreeMemAfterTest)
            {
                fail("** When using a offical version of OJB, ignore this failure! **" +
                        " Memory usage after this test differs more than "+(100 - minimalFreeMemAfterTest)
                        +"% from beginning, this may indicate" +
                        " a memory leak (GC can't free unused metadata objects), but this could also be a result" +
                        " of your JVM settings. Please re-run test.");
            }
        }
        finally
        {
            if (broker != null) broker.close();
        }
    }

    private long convertToMB(long byteValue)
    {
        return (byteValue / 1024) / 1024;
    }

    // ======================================================================
    // inner test class
    // ======================================================================
    class ThreadedUsingBroker extends JUnitExtensions.MultiThreadedTestCase.TestCaseRunnable
    {
        int loops;
        String title = "ThreadedUsingBroker_" + System.currentTimeMillis();

        public ThreadedUsingBroker()
        {
        }

        public ThreadedUsingBroker(int loops)
        {
            this.loops = loops;
        }

        public void runTestCase() throws Exception
        {
            MetadataManager mm = MetadataManager.getInstance();
            DescriptorRepository dr = mm.copyOfGlobalRepository();
            ClassDescriptor cld = dr.getDescriptorFor(targetTestClass);
            // we change a class descriptor value
            cld.setTableName(newTestObjectString);
            // set the changed repository for this thread
            mm.setDescriptor(dr);

            int k = 0;
            while (k < loops)
            {
                PersistenceBroker broker = null;
                try
                {
                    broker = PersistenceBrokerFactory.defaultPersistenceBroker();
                    cld = broker.getClassDescriptor(targetTestClass);
                    String name = cld.getFullTableName();
                    assertEquals(newTestObjectString, name);
                    assertTrue(MetadataManager.getInstance().isEnablePerThreadChanges());
                }
                finally
                {
                    if (broker != null) broker.close();
                }

                try
                {
                    broker = PersistenceBrokerFactory.defaultPersistenceBroker();
                    // check made changes
                    cld = broker.getClassDescriptor(targetTestClass);
                    String name = cld.getFullTableName();
                    assertEquals(newTestObjectString, name);
                    assertTrue(MetadataManager.getInstance().isEnablePerThreadChanges());

                    // query a test object
                    Query query = new QueryByCriteria(Person.class, null, true);
                    broker.getCollectionByQuery(query);
                    // store target object
                    /*
                    store some complex objects to check if references to
                    metadata classes are cached
                    */
                    Project project = new Project();
                    project.setTitle(title);

                    Person p1 = new Person();
                    p1.setFirstname(title);
                    List l1 = new ArrayList();
                    l1.add(project);
                    p1.setProjects(l1);

                    Person p2 = new Person();
                    p2.setFirstname(title);
                    List l2 = new ArrayList();
                    l2.add(project);
                    p2.setProjects(l2);

                    Role r1 = new Role();
                    r1.setPerson(p1);
                    r1.setRoleName(title);
                    r1.setProject(project);
                    List roles1 = new ArrayList();
                    roles1.add(r1);

                    Role r2 = new Role();
                    r2.setPerson(p2);
                    r2.setRoleName(title);
                    r2.setProject(project);
                    List roles2 = new ArrayList();
                    roles2.add(r2);

                    p1.setRoles(roles1);
                    p2.setRoles(roles2);

                    Object obj = ClassHelper.newInstance(targetTestClass);

                    broker.beginTransaction();
                    broker.store(obj);
                    broker.store(p1);
                    broker.store(p2);
                    broker.commitTransaction();
                    // delete target object
                    broker.beginTransaction();
                    broker.delete(obj);
                    //broker.delete(p1);
                    //broker.delete(p2);
                    broker.commitTransaction();
                }
                finally
                {
                    if (broker != null) broker.close();
                }

                k++;
                try
                {
                    Thread.sleep(5);
                }
                catch (InterruptedException e)
                {
                }
            }

        }
    }

    // ======================================================================
    // inner test class
    // ======================================================================
    class GlobalUsingBroker extends JUnitExtensions.MultiThreadedTestCase.TestCaseRunnable
    {
        int loops;

        public GlobalUsingBroker(int loops)
        {
            this.loops = loops;
        }

        public void runTestCase()
        {
            PersistenceBroker broker = null;
            int k = 0;
            try
            {
                while (k < loops)
                {
                    try
                    {
                        MetadataManager.getInstance().setDescriptor(defaultRepository);
                        broker = PersistenceBrokerFactory.defaultPersistenceBroker();
                        ClassDescriptor cld = broker.getClassDescriptor(targetTestClass);
                        assertTrue(MetadataManager.getInstance().isEnablePerThreadChanges());
                        String name = cld.getFullTableName();
                        // this PB instance use unchanged global metadata repository
                        assertEquals(getTestObjectString(), name);
                    }
                    finally
                    {
                        if (broker != null) broker.close();
                    }
                    try
                    {
                        broker = PersistenceBrokerFactory.defaultPersistenceBroker();
                        ClassDescriptor cld = broker.getClassDescriptor(targetTestClass);
                        assertTrue(MetadataManager.getInstance().isEnablePerThreadChanges());
                        String name = cld.getFullTableName();
                        // this PB instance use unchanged global metadata repository
                        assertEquals(getTestObjectString(), name);
                        // System.out.println("Default: found "+name);

                        // query a test object
                        Query query = new QueryByCriteria(Person.class, null, true);
                        broker.getCollectionByQuery(query);
                        // store target object
                        Object obj = ClassHelper.newInstance(targetTestClass);
                        broker.beginTransaction();
                        broker.store(obj);
                        broker.commitTransaction();
                        // delete target object
                        broker.beginTransaction();
                        broker.delete(obj);
                        broker.commitTransaction();
                    }
                    finally
                    {
                        if (broker != null) broker.close();
                    }

                    k++;
                    try
                    {
                        Thread.sleep(5);
                    }
                    catch (InterruptedException e)
                    {
                    }
                }
            }
            catch (Exception e)
            {
                e.printStackTrace();
                throw new OJBRuntimeException(e);
            }
        }
    }
}
