package org.apache.ojb.junit;

import junit.framework.TestCase;

/**
 * Extension of the JUnit test class.
 *
 * @author <a href="mailto:arminw@apache.org">Armin Waibel</a>
 * @version $Id: OJBTestCase.java,v 1.5 2004/06/24 13:43:41 arminw Exp $
 */
public class OJBTestCase extends TestCase
{
    private static final String SKIP_STR = "OJB.skip.issues";
    private static final String SKIP_DEFAULT_VALUE = "false";

    public OJBTestCase()
    {
    }

    public OJBTestCase(String name)
    {
        super(name);
    }

    protected void setUp() throws Exception
    {
        super.setUp();
        // sleep thread to guarantee different timestamp values for
        // each test
        ojbSleep();
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
        // sleep thread to guarantee different timestamp values for
        // each test
        ojbSleep();
    }

    /**
     * Sleep current thread for a minimal period.
     */
    public void ojbSleep()
    {
        try
        {
            Thread.sleep(5);
        }
        catch (InterruptedException ignore)
        {
        }
    }

    /**
     * This method could be used to skip 'problematic' test cases or known issues before
     * a release was made. To enable the skipped tests set a system property 'skip.issues'
     * to <tt>false</tt>.
     */
    public boolean ojbSkipKnownIssueProblem()
    {
        return skipKnownIssueProblem(null);
    }

    /**
     * This method could be used to skip 'problematic' test cases or known issues before
     * a release was made. To enable the skipped tests set a system property 'skip.issues'
     * to <tt>false</tt>.
     */
    public boolean skipKnownIssueProblem(String message)
    {
        String result = SKIP_DEFAULT_VALUE;
        boolean skip = false;
        try
        {
            result = System.getProperty(SKIP_STR, result);
            skip = new Boolean(result).booleanValue();
        }
        catch(Exception e)
        {
            System.err.println("Seems that system property '" + SKIP_STR + "=" + result + "' is not a valid boolean value");
        }
        if(skip)
        {
            if(message == null)
            {
               message = "Will skip a known issue";
            }
            String className = this.getClass().getName();
            System.out.println("# [Skip test in " + className + "] " + message + " #");
        }
        return skip;
    }
}
