/*
 * Created by IntelliJ IDEA.
 * User: Matt
 * Date: Jun 10, 2002
 * Time: 9:22:36 PM
 * To change template for new class use
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package org.apache.ojb.odmg;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.apache.ojb.broker.Article;
import org.apache.ojb.broker.InterfaceArticle;
import org.apache.ojb.broker.InterfaceProductGroup;
import org.apache.ojb.broker.Mammal;
import org.apache.ojb.broker.ProductGroup;
import org.apache.ojb.broker.ProductGroupProxy;
import org.apache.ojb.broker.Reptile;
import org.apache.ojb.broker.metadata.ClassDescriptor;
import org.apache.ojb.broker.metadata.MetadataManager;
import org.apache.ojb.junit.ODMGTestCase;
import org.apache.ojb.odmg.shared.ODMGZoo;
import org.odmg.ODMGException;
import org.odmg.OQLQuery;
import org.odmg.Transaction;

public class OneToManyTest extends ODMGTestCase
{
    private static final int COUNT = 10;
    int groupID = 3300;
    int articleID = 2200;

    public static void main(String[] args)
    {
        String[] arr = {OneToManyTest.class.getName()};
        junit.textui.TestRunner.main(arr);
    }

    public OneToManyTest(String name)
    {
        super(name);
    }

    public void setUp() throws Exception
    {
        super.setUp();
        ClassDescriptor cld = MetadataManager.getInstance().getRepository().getDescriptorFor(ProductGroup.class);
        cld.getCollectionDescriptorByName("allArticlesInGroup").setCascadeStore(false);
    }

    public void tearDown() throws Exception
    {
        ClassDescriptor cld = MetadataManager.getInstance().getRepository().getDescriptorFor(ProductGroup.class);
        cld.getCollectionDescriptorByName("allArticlesInGroup").setCascadeStore(false);
        super.tearDown();
    }

    /**
     * tests creation of new object that has a one to many relationship
     *
     * @throws Exception
     */
    public void notYetCorrectTestCreate()
            throws Exception
    {
        /**
         * 1. create the article.
         */
        int id = groupID;
        Transaction tx = odmg.newTransaction();
        tx.begin();
        ProductGroup group = new ProductGroup();
        tx.lock(group, Transaction.WRITE);
        group.setId(id);
        group.setGroupName("id" + id);
        for (int i = 0; i < COUNT; i++)
        {
            Article article = createArticle(i);
            group.add(article);
        }
        tx.commit();
        /**
         * 2. query on the article to make sure it is everything we set it up to be.
         */
        OQLQuery query = odmg.newOQLQuery();
        query.create("select productGroup from " + ProductGroup.class.getName() + " where groupName=$1");
        query.bind("id" + id);
        Collection results = (Collection) query.execute();
        Iterator it = results.iterator();
        int foundCount = 0;
        ProductGroupProxy temp = null;
        while (it.hasNext())
        {
            foundCount++;
            temp = (ProductGroupProxy) it.next();
            if (temp.getAllArticles().size() != COUNT)
                fail("ProductGroup should have had " + COUNT + " articles in it, we found " + temp.getAllArticles().size());
        }
        if (foundCount != 1)
            fail("Should have found just one ProductGroup with id=id" + id + ", instead found " + foundCount);
    }

    /**
     * tests creation of new object that has a one to many relationship.
     * thma: thhis test will not work, because ODMG is no able to track
     * modifictations to normal collections.
     * Only Odmg Collections like DList will be treated properly.
     *
     * @throws Exception
     */
    public void brokenTestUpdateWithProxy()
            throws Exception
    {
        Transaction tx = odmg.newTransaction();
        tx.begin();
        /**
         * 1. get all articles from groups and add a new article.
         */
        OQLQuery query = odmg.newOQLQuery();
        query.create("select productGroup from " + ProductGroup.class.getName());
        Collection results = (Collection) query.execute();
        Iterator it = results.iterator();
        InterfaceProductGroup temp = null;
        /**
         * to each productgroup add an article with a pre-determined name.
         */

        int i = 2300;
        while (it.hasNext())
        {
            Article article = createArticle(i++);
            article.setArticleName("OneToManyUpdateTest");
            temp = (InterfaceProductGroup) it.next();
            tx.lock(temp, Transaction.WRITE);
            temp.add(article);
            article.setProductGroup(temp);
        }
        tx.commit(); // thma: during this commit no article is persisted! works as designed!

        /**
         * 2. requery and find the articles we added.
         */
        query = odmg.newOQLQuery();
        query.create("select productGroup from " + ProductGroup.class.getName());
        results = (Collection) query.execute();
        it = results.iterator();
        temp = null;
        boolean found = false;
        while (it.hasNext())
        {
            found = false;
            temp = (InterfaceProductGroup) it.next();
            Collection articles = temp.getAllArticles();
            Iterator it2 = articles.iterator();
            while (it2.hasNext())
            {
                InterfaceArticle art = (InterfaceArticle) it2.next();
                if (art.getArticleName() != null)
                {
                    if (art.getArticleName().equals("OneToManyUpdateTest"))
                    {
                        found = true;
                    }
                }
            }
            if (!found)
                fail("Should have found article with name OneToManyUpdateTest");
        }
    }

    /**
     * this tests if polymorph collections (i.e. collections of objects
     * implementing a common interface) are treated correctly
     */
    public void testPolymorphOneToMany()
    {

        ODMGZoo myZoo = new ODMGZoo("London");
        Mammal elephant = new Mammal(37, "Jumbo", 4);
        Mammal cat = new Mammal(11, "Silvester", 4);
        Reptile snake = new Reptile(3, "Kaa", "green");

        myZoo.addAnimal(snake);
        myZoo.addAnimal(elephant);
        myZoo.addAnimal(cat);

        try
        {
            Transaction tx = odmg.newTransaction();
            tx.begin();
            database.makePersistent(myZoo);
            tx.commit();

            int id = myZoo.getZooId();

            tx = odmg.newTransaction();
            tx.begin();
            OQLQuery query = odmg.newOQLQuery();
            query.create("select zoos from " + ODMGZoo.class.getName() +
                    " where zooId=$1");
            query.bind(new Integer(id));
            List zoos = (List) query.execute();
            assertEquals(1, zoos.size());
            ODMGZoo zoo = (ODMGZoo) zoos.get(0);
            tx.commit();
            assertEquals(3, zoo.getAnimals().size());


        }
        catch (ODMGException e)
        {
            e.printStackTrace();
            fail("ODMGException thrown " + e.getMessage());
        }
    }

    /**
     * Create an article with 4 product groups related to it in a 1-N relationship
     *
     * @param id
     * @return
     */
    protected Article createArticle(int id)
    {
        Article a = Article.createInstance();
        a.setArticleId(id);
        a.setArticleName("New ODMG Article " + id);
        a.setIsSelloutArticle(true);
        a.setMinimumStock(100);
        a.setOrderedUnits(17);
        a.setPrice(0.45);
        //a.setProductGroupId(1);
        a.setStock(234);
        a.setSupplierId(4);
        a.setUnit("bottle");
        return a;
    }
}
