package xdoclet.modules.ojb.tests;

/* Copyright 2003-2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Tests for the ojb.class tag with the generate-table-info attribute.
 *
 * @author <a href="mailto:tomdz@users.sourceforge.net">Thomas Dudziak (tomdz@users.sourceforge.net)</a>
 */
public class ClassTagGenerateTableInfoAttributeTests extends OjbTestBase
{
    public ClassTagGenerateTableInfoAttributeTests(String name)
    {
        super(name);
    }

    // Test of generate-table-info: empty value
    public void testGenerateTableInfo1()
    {
        addClass(
            "A",
            "/** @ojb.class generate-table-info=\"\" */\n"+
            "public class A {}\n");

        assertNull(runOjbXDoclet(OJB_DEST_FILE));
        assertNull(runTorqueXDoclet(TORQUE_DEST_FILE, "ojbtest"));
    }

    // Test of generate-table-info: invalid value
    public void testGenerateTableInfo2()
    {
        addClass(
            "A",
            "/** @ojb.class generate-table-info=\"yes\" */\n"+
            "public class A {}\n");

        assertNull(runOjbXDoclet(OJB_DEST_FILE));
        assertNull(runTorqueXDoclet(TORQUE_DEST_FILE, "ojbtest"));
    }

    // Test of generate-table-info: 'true' value
    public void testGenerateTableInfo3()
    {
        addClass(
            "A",
            "/** @ojb.class generate-table-info=\"true\" */\n"+
            "public class A {}\n");

        assertEqualsOjbDescriptorFile(
            "<class-descriptor\n"+
            "    class=\"A\"\n"+
            "    table=\"A\"\n"+
            ">\n"+
            "</class-descriptor>",
            runOjbXDoclet(OJB_DEST_FILE));
        assertEqualsTorqueSchemaFile(
            "<database name=\"ojbtest\">\n"+
            "    <table name=\"A\">\n"+
            "    </table>\n"+
            "</database>",
            runTorqueXDoclet(TORQUE_DEST_FILE, "ojbtest"));
    }

    // Test of generate-table-info: 'true' value with remote base- and subclass
    public void testGenerateTableInfo4()
    {
        addClass(
            "A",
            "/** @ojb.class */\n"+
            "public class A {}\n");
        addClass(
            "B",
            "public class B extends A {}\n");
        addClass(
            "C",
            "/** @ojb.class generate-table-info=\"true\" */\n"+
            "public class C extends B {}\n");
        addClass(
            "D",
            "public class D extends C {}\n");
        addClass(
            "E",
            "/** @ojb.class */\n"+
            "public class E extends D {}\n");

        assertEqualsOjbDescriptorFile(
            "<class-descriptor\n"+
            "    class=\"A\"\n"+
            "    table=\"A\"\n"+
            ">\n"+
            "    <extent-class class-ref=\"C\"/>\n"+
            "</class-descriptor>\n"+
            "<class-descriptor\n"+
            "    class=\"C\"\n"+
            "    table=\"C\"\n"+
            ">\n"+
            "    <extent-class class-ref=\"E\"/>\n"+
            "</class-descriptor>\n"+
            "<class-descriptor\n"+
            "    class=\"E\"\n"+
            "    table=\"E\"\n"+
            ">\n"+
            "</class-descriptor>",
            runOjbXDoclet(OJB_DEST_FILE));
        assertEqualsTorqueSchemaFile(
            "<database name=\"ojbtest\">\n"+
            "    <table name=\"A\">\n"+
            "    </table>\n"+
            "    <table name=\"C\">\n"+
            "    </table>\n"+
            "    <table name=\"E\">\n"+
            "    </table>\n"+
            "</database>",
            runTorqueXDoclet(TORQUE_DEST_FILE, "ojbtest"));
    }

    // Test of generate-table-info: 'false' value
    public void testGenerateTableInfo5()
    {
        addClass(
            "A",
            "/** @ojb.class generate-table-info=\"false\" */\n"+
            "public class A {}\n");

        assertEqualsOjbDescriptorFile(
            "<class-descriptor\n"+
            "    class=\"A\"\n"+
            ">\n"+
            "</class-descriptor>",
            runOjbXDoclet(OJB_DEST_FILE));
        assertEqualsTorqueSchemaFile(
            "<database name=\"ojbtest\">\n"+
            "</database>",
            runTorqueXDoclet(TORQUE_DEST_FILE, "ojbtest"));
    }

    // Test of generate-table-info: 'false' value with remote base- and subclass
    public void testGenerateTableInfo6()
    {
        addClass(
            "A",
            "/** @ojb.class */\n"+
            "public class A {}\n");
        addClass(
            "B",
            "public class B extends A {}\n");
        addClass(
            "C",
            "/** @ojb.class generate-table-info=\"false\" */\n"+
            "public class C extends B {}\n");
        addClass(
            "D",
            "public class D extends C {}\n");
        addClass(
            "E",
            "/** @ojb.class */\n"+
            "public class E extends D {}\n");

        assertEqualsOjbDescriptorFile(
            "<class-descriptor\n"+
            "    class=\"A\"\n"+
            "    table=\"A\"\n"+
            ">\n"+
            "    <extent-class class-ref=\"C\"/>\n"+
            "</class-descriptor>\n"+
            "<class-descriptor\n"+
            "    class=\"C\"\n"+
            ">\n"+
            "    <extent-class class-ref=\"E\"/>\n"+
            "</class-descriptor>\n"+
            "<class-descriptor\n"+
            "    class=\"E\"\n"+
            "    table=\"E\"\n"+
            ">\n"+
            "</class-descriptor>",
            runOjbXDoclet(OJB_DEST_FILE));
        assertEqualsTorqueSchemaFile(
            "<database name=\"ojbtest\">\n"+
            "    <table name=\"A\">\n"+
            "    </table>\n"+
            "    <table name=\"E\">\n"+
            "    </table>\n"+
            "</database>",
            runTorqueXDoclet(TORQUE_DEST_FILE, "ojbtest"));
    }
}
